from copy import deepcopy
import curves as c
from sys import stderr
#from micc.cgraph import cdfs


def shift(path):
    '''
    init

    '''
    temp = path.index(min(path))
    return path[temp:] + path[:temp]


def invert(path):
    '''
    init

    '''
    return shift(path[::-1])


def contains(small, big):
    print type(small), type(big)
    for i in xrange(len(big)-len(small)+1):
        for j in xrange(len(small)):
            if big[i+j] != small[j]:
                break
        else:
            return i, i+len(small)
    return False


class Graph:

    def add_node(self, node):
        self.nodes[node] = []

    def __init__(self, edges, rep_num=2):
        self.edges = edges
        self.rep_num = rep_num
        self.nodes = {}
        self.counter = 0
        self.loops = []
        self.gammas = []
        self.nodes_to_faces = {}
        self.rep_num = rep_num

    def compute_loops(self, n, genus):
        edges = self.edges[0]
        fourgons = [i[1] for i in edges if i[0] == 4]
        non_fourgons = [i[1] for i in edges if i[0] != 4]
        keys = self.nodes_to_faces.keys()
        for i,face in enumerate(non_fourgons):
            for node in face:
                if not node in keys:
                    self.nodes_to_faces[node] = [i]
                else:
                    self.nodes_to_faces[node].append(i)
                keys = self.nodes_to_faces.keys()
        for face in fourgons:
            for node in face:
                if not node in keys:
                    self.nodes_to_faces[node] = [None]
                else:
                    self.nodes_to_faces[node].append(None)

                keys = self.nodes_to_faces.keys()
        self.nodes_to_faces = {int(k): tuple(v) for k,v in self.nodes_to_faces.iteritems()}
        nodes = range(n)

        for i in nodes:
            self.add_node(i)
        self.find_all_edges(fourgons, non_fourgons, nodes, self.rep_num)
        graph_copy = deepcopy(self.nodes)
        #stderr.write(str(edges)+'\n')
        #stderr.write(str(graph_copy)+'\n')
        #graph_copy = {i : set(j) for i,j in self.nodes.iteritems()}
        #for k,v in graph_copy.iteritems():
        #    stderr.write(str(k)+": "+str(v)+'\n')
        #raw_input()
        #from sys import stderr
        #stderr.write(str(graph_copy)+'\n')
        #stderr.write(str(self.nodes_to_faces)+'\n')
        #self.loops = cdfs(0,0,graph_copy,[], self.nodes_to_faces)
        for start_node in nodes:
            #self.loops.extend(self.iter_loop_dfs(graph_copy, start_node, start_node))
            for adj_node in graph_copy[start_node]:
                #print start_node,adj_node,graph_copy,[start_node], self.nodes_to_faces
                self.loops += loop_dfs(start_node,adj_node,graph_copy,[start_node], self.nodes_to_faces)
                #self.loops += self.iter_loop_dfs(graph_copy, start_node, start_node,self.nodes_to_faces)
        '''
        #Johnson circuit locating algorithm
        from johnson import Johnson
        johnny = Johnson(graph_copy)
        johnny.find_all_circuits()
        self.loops = johnny.circuits
        '''
        #print len(self.loops)
        from itertools import chain

        self.loops = [j for j in set([tuple(i) for i in self.loops])]
        edges = self.edges[1]
        for path in list(self.loops):
            temp_len = len(path)
            #in_loops = path in set(self.loops)
            removed = False
            if temp_len < 3:
                if not removed:
                    self.loops.remove(path)
                    removed = True

            elif invert(path) in self.loops:
                if not removed:
                    self.loops.remove(path)
                    removed = True

            # Trial: remove all duplicates
            else:
                temp_path = list(path)
                #temp_len = len(temp_path)
                temp_path = shift(temp_path)
                for face in non_fourgons:
                    for triple in [temp_path[i:i+3] \
                            for i in xrange(temp_len-2)]:
                        if set(triple) <= face:
                            if not removed:
                                self.loops.remove(path)
                                removed = True
                                break
                    if removed:
                        break


                    temp_path = invert(temp_path)

                    for triple in [temp_path[i:i+3] \
                            for i in xrange(temp_len-2)]:
                        if set(triple) <= face:
                            if not removed:
                                self.loops.remove(path)
                                removed = True
                                break

                    for i in xrange(temp_len):
                        temp_path = temp_path[1:] + temp_path[:1]
                        for triple in (temp_path[i:i+3] for i in xrange(temp_len-2)):
                            boolA = set(triple) <= face

                            #boolA = contains(triple,face)
                            #if set(triple) <= set(face) and path in self.loops:
                            if boolA :
                                if not removed:
                                    self.loops.remove(path)
                                    removed = True
                                    break
                        if removed:
                            break

        for loop in list(self.loops):
            path = list(loop)
            path_matrix = c.build_matrices(edges, [path])
            ladder = [list(path_matrix[0][0,:,1]),list(path_matrix[0][0,:,3])]

            gamma = c.CurvePair(ladder[0],ladder[1],0, 0)
            if gamma.genus <= genus:
                self.gammas.append(loop)

    @staticmethod
    def get_value(pos_to_insert, ladder, path):
        return int(path[int(ladder[int(pos_to_insert)])])+1

    def find_all_edges(self, fourgons, non_fourgons, alpha_edge_nodes, rep_num):
        '''
        Determines all edges between boundary componenets and adds adjacencies between
        appropriate edges.

        '''

        #find all direct connections between non-fourgon regions
        regions = fourgons + non_fourgons
        for alpha_edge in alpha_edge_nodes:
            for region in regions:
                if alpha_edge in region:
                    region.remove(alpha_edge)
                    for other_edge in region:
                        self.add_adjacency(alpha_edge, int(other_edge), rep_num)
                    region.add(alpha_edge)

    def add_adjacency(self,node, adjacent_node, rep_num):
        '''
        Adds adjacencies to a graph represented by an adjacency list.
        This is useful when we would like to replicate adjacencies
        between nodes.

        :param self:
        :type self: Graph
        :param node: node to add adjacency
        :param type: int
        :param adjacent_node: the adjacent node
        :type adjacent_node: int
        :param rep_num: set to 1
        :type rep_num: int

        '''
        for i in range(rep_num):
            adjacency_list = self.nodes[node]
            if Graph.count(adjacent_node, adjacency_list) < rep_num:
                adjacency_list.append(adjacent_node)

            adjacency_list = self.nodes[adjacent_node]
            if Graph.count(node, adjacency_list) < rep_num:
                adjacency_list.append(node)

    @staticmethod
    def count(adj_node, adj_list):
        '''
        Determines the number of adjacencies between two nodes in a graph, given the
        adjacency list of one of node.

        :param self:
        :type self: Graph
        :param adj_node: Adjacent node
        :type adj_node: int
        :param adj_list: Adjacency list of the node in question
        :type adj_list: list<int>
        :returns: number of edges from the adjacent node to the original node


        count = 0
        for i in adj_list:
            if i == adj_node:
                count += 1
        return count
        '''
        return adj_list.count(adj_node)





    @staticmethod
    def faces_share_edges(nodes_to_faces, path):
        path_head_3 = path[-3:]
        previous_three_faces = [nodes_to_faces[edge] for edge in path_head_3]
        previous_three_faces = [set(i) for i in previous_three_faces]
        intersection_all = set.intersection(*previous_three_faces)
        return len(intersection_all) == 2

    def iter_loop_dfs(self, graph, start,  goal, nodes_to_faces):
        loops = []
        stack = [(start, [start])]
        while stack:
            vertex, path = stack.pop()
            in_path = set(path)
            for next in graph[vertex]:
                if next in in_path:
                    if len(path) >= 3:
                        if Graph.faces_share_edges(nodes_to_faces, path):
                            continue
                    if next == goal:
                        loops.append(list(path))
                    else:
                        continue
                else:
                    stack.append((next, list(path + [next])))
        return loops


from sys import stderr
def loop_dfs( current_node,  start_node,  graph,  current_path,  nodes_to_faces):
    '''
    Recursively finds all closed cycles in a given graph that begin and end at start_node.
    As one would guess, it employs a standard depth-first search algorithm on the graph,
    appending current_path to all_loops when it returns to start_node.

    In the overall distance computation, this function is computationally dominating with
    exponential complexity, so take care with its use.

    :param self:
    :type self: Graph
    :param current_node: the current alpha edge in the recursion
    :type current_node: int
    :param start_node: the source node of the current recursive search
    :type start_node: int
    :param graph: graph of the overall graph mid-recursion
    :type graph: dict<int,list<int> >
    :param current_path: list of nodes in the current path
    :type current_path: list<int>
    :param all_loops: list of all current paths
    :type all_loops: list< list<int> >
    :returns: set of all closeds cycles in the graph starting and ending at start_node

    '''
    if len(current_path) >= 3:
        path_head_3 = current_path[-3:]
        previous_three_faces = [set(nodes_to_faces[edge]) for edge in path_head_3]
        intersection_all = set.intersection(*previous_three_faces)
        if len(intersection_all) == 2:
            return []

    if current_node == start_node:
        #stderr.write("Found one! \n")
        #all_loops.append(shift(list(current_path)))
        return [shift(list(current_path))]

    else:
        loops = []
        for adjacent_node in set(graph[current_node]):
            if Graph.count(adjacent_node, current_path) < 1:
                current_path.append(adjacent_node)
                graph[current_node].remove(adjacent_node)
                graph[adjacent_node].remove(current_node)
                loops += list(loop_dfs(adjacent_node, start_node, graph, current_path, nodes_to_faces))
                graph[current_node].append(adjacent_node)
                graph[adjacent_node].append(current_node)
                current_path.pop()
        return loops
'''
for i in range(len(path)):
    index = path[i]
    beta[0].insert(index+1,None)
    beta[1].insert(index+1,None)
    beta[0].append('____')
    beta[1].append('____')
    for j,tup in enumerate(zip(beta[0],beta[1])):

        t,b = tup
        if type(t) == str or type(b) == str: continue
        if t > index:
            beta[0][j] +=1
        if b > index:
            beta[1][j] +=1
    beta[0].remove('____')
    beta[1].remove('____')


    for j in range(len(path)):
        if path[j] > index:
            path[j] += 1
path.sort()
for i in range(len(path)):
    index = path[i]
    beta[0][index+1]=int(self.getValue(i,ladder[0],path))
    beta[1][index+1] =int(self.getValue(i,ladder[1],path))
'''
'''
count = {}
count[None] = 0
for i,j in zip(beta[0],beta[1]):
    if i in count.keys():
        count[i] += 1
    else:
        count[i] = 1
    if j in count.keys():
        count[j] += 1
    else:
        count[j] = 1
'''



from copy import deepcopy
from sys import maxint
index = 0

class Johnson:

    def __init__(self, graph):
        self.n = len(graph.keys())
        self.k = 0
        self.graph = graph
        self.circuits = []
        self.blocked = {}
        self.blocked_nodes = {}

    def circuit(self, subgraph, v, s, stack):
        if len(subgraph.keys()) is 0:
            return False

        f = False
        stack.append(v)
        self.blocked[v] = True

        for w in self.graph[v]:
            if w is s:
                #stack.append(s)
                self.circuits.append(deepcopy(stack))
                #stack.pop()
                f = True
            else:
                if not self.blocked[w]:
                    if self.circuit(subgraph, w, s, stack):
                        f = True

        if f:
            self.unblock(v)
        else:
            for w in self.graph[v]:
                if v not in self.blocked_nodes[w]:
                    self.blocked_nodes[w].append(v)

        stack.pop()
        return f

    def unblock(self, u):
        self.blocked[u] = False

        while len(self.blocked_nodes[u]) > 0:
            w = self.blocked_nodes[u].pop(0)
            if self.blocked[w]:
                self.unblock(w)

    @staticmethod
    def least_vertex(graph):
        return min(graph.keys())

    @staticmethod
    def subgraph_from_vertex(i, graph):
        result = {j : [] for j in graph.keys()}
        for end_point in graph.keys():
            if end_point >= i:
                for start_point in graph[end_point]:
                    if start_point >= i:
                        result[start_point].append(end_point)
                        result[end_point].append(start_point)

        return result

    @staticmethod
    def least_scc(graph):
        def get_sccs(graph):
            sccs = []
            index = 0
            stack = []
            index_map = {}
            low_link_map = {}

            def strongly_connected(v):
                global index
                index_map[v] = index
                low_link_map[v] = index
                index += 1
                stack.append(v)
                for w in graph[v]:
                    if w not in index_map:
                        strongly_connected(w)
                        low_link_map[v] = min([low_link_map[v], low_link_map[w]])
                    else:
                        if w in stack:
                            low_link_map[v] = min([low_link_map[v], index_map[w]])

                result = []
                if low_link_map[v] == index_map[v]:
                    scc_list = []
                    while True:
                        w = stack.pop()
                        scc_list.append(w)
                        if w == v:
                            break
                    if len(scc_list) > 1:
                        result.append(scc_list)
                return result

            for v in graph.keys():
                if v not in index_map:
                    sccs += strongly_connected(v)
            return sccs

        def add_edges(min_scc, graph):  #remove this and make a generic funciton to call
            result = {i: [] for i in min_scc}
            for i in min_scc:
                for edge in [(i, j) for j in graph[i]]:
                    to = edge[1]
                    if to in min_scc:
                        result[i].append(to)
                        result[to].append(i)
            return result

        sccs = get_sccs(graph)
        minint = maxint
        min_scc = []
        for scc in sccs:
            if len(scc) == 1:
                continue
            for i in scc:
                if i < minint:
                    min_scc = scc
                    minint = i
        return add_edges(min_scc, graph)

    def find_all_circuits(self):
        #self.blocked = {i : False for i in range(self.n)}
        #self.blocked_nodes = {i : [] for i in range(self.n)}
        stack = []
        s = 0  #possibly zero in this setting
        while s < self.n:
            subgraph = Johnson.subgraph_from_vertex(s, self.graph)
            least_scc = Johnson.least_scc(subgraph)
            if len(least_scc.keys()) > 0:
                s = Johnson.least_vertex(least_scc)
                for i in least_scc.keys():
                    self.blocked[i] = False
                    self.blocked_nodes[i] = []
                self.circuit(least_scc, s, s, stack)
                s += 1
            else:
                s = self.n
if __name__ == '__main__':
    j = Johnson({
        1: [2, 6],
        2: [3, 1],
        3: [4, 2, 6],
        4: [5, 3],
        5: [6, 4],
        6: [5, 1, 4]
    })

    j.find_all_circuits()
    print j.circuits


"""
This is NOT for use to build releases, but rather for compiling via Cython

Usage:
    python setup.py build_ext --inplace
"""

from distutils.core import setup
from Cython.Build import cythonize

setup(
    name='MICC',
    ext_modules=cythonize(["graph.py"])
)




from ctypes import cdll, byref, py_object,c_float, c_int, Structure
import subprocess
from json import dumps,loads

def shift(path):
    '''
    init

    '''
    temp = path.index(min(path))
    return path[temp:] + path[:temp]

def loop_dfs( current_node,  start_node,  graph,  current_path,  nodes_to_faces):
    if len(current_path) >= 3:
        path_head_3 = current_path[-3:]
        previous_three_faces = [set(nodes_to_faces[edge]) for edge in path_head_3]
        intersection_all = set.intersection(*previous_three_faces)
        if len(intersection_all) == 2:
            return []

    if current_node == start_node:
        #stderr.write("Found one! \n")
        #all_loops.append(shift(list(current_path)))
        return [shift(list(current_path))]

    else:
        loops = []
        for adjacent_node in set(graph[current_node]):
            if current_path.count(adjacent_node) < 1:
                current_path.append(adjacent_node)
                graph[current_node].remove(adjacent_node)
                graph[adjacent_node].remove(current_node)
                loops += list(loop_dfs(adjacent_node, start_node, graph, current_path, nodes_to_faces))
                graph[current_node].append(adjacent_node)
                graph[adjacent_node].append(current_node)
                current_path.pop()
        return loops

#lib.loop_dfs(0,0,py_object({1:'hello'}),py_object([]),py_object({}))
to_cpp = [0, 8, {0: [11, 8, 15], 1: [9, 12], 2: [10, 13, 15], 3: [11, 6, 14],
                              4: [12, 7], 5: [13, 8], 6: [3, 9, 14], 7: [4, 16, 10], 8: [0, 5],
                              9: [1, 6], 10: [2, 7, 16], 11: [0, 3, 15], 12: [1, 4],
                              13: [2, 5, 15], 14: [3, 6, 16], 15: [0, 2, 11, 13], 16: [7, 10, 14]}, [0],
                               {0: (0, None), 1: (None, None), 2: (1, None), 3: (2, None),
                                    4: (None, None), 5: (None, None), 6: (2, None), 7: (3, None),
                                    8: (None, None), 9: (None, None), 10: (3, None), 11: (0, None),
                                    12: (None, None), 13: (1, None), 14: (2, None), 15: (0, 1),16: (3, None)}]

'''
l = loop_dfs(0, 8, {0: [11, 8, 15], 1: [9, 12], 2: [10, 13, 15], 3: [11, 6, 14],
                      4: [12, 7], 5: [13, 8], 6: [3, 9, 14], 7: [4, 16, 10], 8: [0, 5],
                      9: [1, 6], 10: [2, 7, 16], 11: [0, 3, 15], 12: [1, 4],
                      13: [2, 5, 15], 14: [3, 6, 16], 15: [0, 2, 11, 13], 16: [7, 10, 14]},[0],
          {0: (0, None), 1: (None, None), 2: (1, None), 3: (2, None),
           4: (None, None), 5: (None, None), 6: (2, None), 7: (3, None),
           8: (None, None), 9: (None, None), 10: (3, None), 11: (0, None),
           12: (None, None), 13: (1, None), 14: (2, None), 15: (0, 1),16: (3, None)})
'''


def cdfs(current_node, start_node, graph, current_path, nodes_to_face):
    to_cpp = [current_node, start_node, current_path, graph, nodes_to_face]
    import os
    j = dumps(to_cpp,separators=(',' , ':'))
    f = open('intermed.json','w')
    f.write(j)
    f.close()
    #os.system("./util-dfs intermed.json")
    #s = subprocess.check_output("./util-dfs "+dumps(to_cpp,separators=(',' , ':')), shell=True)
    s = subprocess.check_output("micc/util-dfs intermed.json", shell=True)
    '''
    from sys import stderr
    stderr.write("FUCK THIS HORSESHIT")
    stderr.write(s)
    '''
    return loads(s)

#cdfs(to_cpp[0], to_cpp[1], to_cpp[2], to_cpp[3], to_cpp[4] )

#s = subprocess.check_output("./util-dfs "+dumps(to_cpp,separators=(',' , ':')), shell=True)

#g = Graph()
#g.bar()




# Give curve pairs class structure in preparation for public access
import numpy as np
from itertools import product, izip
from copy import deepcopy
import re

def fix_matrix_signs(M):
    '''
    :param M: The matrix with incorrect signs.
    :type M: numpy.array(dtype=float64) of shape (2,n,4)
    :returns: The matrix of shape (2,n,4) with appropriate signs.

    Given a matrix corresponding to a curve pair,
    fixes the signs so that the matrix can be traversed by function bdycount.
    Starts from the simples point, then assigns based on matching values and
    ENS: No qualitative results about the curve pair will be changed by this,
    since orientation is just a formality.

    '''

    row, col = [-1, 1]
    sgn = 1
    #for ii in range(len(M[0,:])):
    while 0 in M[1, :, 1:4:2] and row < M.shape[1]-1:
        row += 1
        sgn = -sgn
        # Starting position and sign are arbitrary.
        for i in range(M.shape[1]):
            g_val = int(M[0, row, col])
            new = np.array(np.where(M[0, g_val, :] == row))
            #print  np.where(new%2==col%2)
            ind = int(new[np.where(new % 2 == col % 2)])
            sgn = -sgn
            M[1, g_val, ind] = sgn
            ind += 2
            ind = ind % M.shape[2]
            sgn = -sgn
            M[1, g_val, ind] = sgn
            row = g_val
            col = ind
        M[1, :, 0] = -1
        M[1, :, 2] = 1
    return M



def concatenate_matrix(M1, M2):
    '''
    :param M1: First matrix to combine
    :type M1: numpy.array(dtype=float64) of shape (2,n,4)
    :param M2: Second matrix to combine
    :type M2: numpy.array(dtype=float64) of shape (2,m,4)
    :returns: final matrix

    Note that n does not have to equal m in general.
    Combines two curve pairs into one.
    REQ: curve pairs are, of course, encoded as matrices.
    INV: new matrix signs will be correct
    INV: final matrix will correspond to a single curve.

    '''
    top_M = np.copy(M1)
    bot_m = np.copy(M2)
    n = M1.shape[1]
    m = M2.shape[1]
    top_M[0, 0, 0] = n + m - 1
    top_M[0, n-1, 2] = n
    bot_m[0] += n * np.ones(M2[0].shape)
    bot_m[0, m-1, 2] = 0
    bot_m[0, 0, 0] = n-1
    new_M = np.vstack((top_M[0], bot_m[0]))  # Multi-curve
    new_M = np.vstack((new_M, np.vstack((top_M[1], bot_m[1])))).reshape((2, n+m, 4))

    left = zip(np.where(top_M[0] == n-1)[0], np.where(top_M[0] == n-1)[1])
    i0, j0 = 0,0
    for pair in left:
        i, j = pair
        if j % 2 == 1:
            i0, j0 = i, j
    new_M[0, i0, j0] = n

    temp_ind = np.where(new_M[0, n-1, :] == i0)[0]
    temp_ind = int(temp_ind[temp_ind % 2 == 1][0])
    temp_val = new_M[0, n-1, temp_ind]
    switch_val = new_M[0, n, temp_ind]
    new_M[0, n-1, temp_ind] = switch_val
    new_M[0, n, temp_ind] = temp_val
    return_ind = new_M[0, switch_val, :] == n
    return_ind[0:3:2] = False
    new_M[0, switch_val, return_ind == True] = n-1

    return fix_matrix_signs(new_M)

## Path finding methods -- used to find edge paths in the complement of beta-curve.
## Most are helpers for findAllPaths below.



def visited(curr, face, path):
    """

    :param curr:
    :param face:
    :param path:
    :return:
    """
    my_face = list(face)
    my_face.remove(curr)
    v = 0
    for edge in my_face:
        if edge in path:
            v = 1
    return v


def is_unique(path, all_paths):
    '''
    :param path:
    :type path:
    :param AllPaths:
    :type AllPaths:
    :returns: boolean

    '''
    return not (path in all_paths)


def shift(path):
    '''
    Reorders path with a minimum intersection as the base point.
    This is useful for determining path uniqueness.

    :param path: list of intersections representing the current path
    :type path: list
    :returns: list with intersections shifted in order such that the lowest intersection is in the first position.

    '''
    temp = path.index(min(path))
    return path[temp:] + path[:temp]


def invert(path):
    '''
    :param path: some path
    :type path: list
    :returns: the inverted path

    '''
    return shift(path[::-1])


def path_finished_single(edge, face, path):
    '''
    :param edge:
    :type edge:
    :param face:
    :type face:
    :param path:
    :type path:
    :returns:

    '''
    C = len(path) > 2 and (edge == path[-1])
    faceL = list(face)
    faceL.remove(edge)
    C = C and not set(faceL).isdisjoint(set(path))
    #meeting = lambda e1, e2, face: e1 in face and e2 in face
    return C


def path_finished_double(edge, face, path):
    '''
    :param edge:
    :type edge:
    :param face:
    :type face:
    :param path:
    :type path:
    :returns:

    '''

    C = len(path) > 2 and (edge == path[-1])
    faceL = list(face)
    faceL.remove(edge)
    C = C and not set(faceL).isdisjoint(set(path))
    if path.count(edge) != 2 :
        C = 0
    #meeting = lambda e1, e2, face: e1 in face and e2 in face
    return C



def find_new_paths(current_path, my_face, faces, all_paths, path_function):
    '''
    :param current_path:
    :type current_path:
    :param my_face:
    :type my_face:
    :param faces:
    :type faces:
    :param all_paths:
    :typeall_pathss:
    :param path_function:
    :typepath_functionn:
    :returns:

    '''

    start = current_path[0]
    next_edge = None
    sub_path = []
    faces_without_current_face = list(faces)
    faces_without_current_face.remove(my_face)
    for face in faces: # Check all faces...
        if start in face and face is not my_face:
            #if we find the start edge in another face...
            for other_face in faces_without_current_face:
                #go through its edges...
                if start in other_face: # needed?
                    face_without_start = list(other_face)
                    face_without_start.remove(start)

                    for non_starting_edge in face_without_start:
                        # try all edges in that path
                        next_edge = non_starting_edge
                        if  (not visited( start, other_face, current_path ) ) and\
                            (next_edge not in my_face):
                            sub_path = [next_edge]
                            sub_path.extend(current_path)

                            # Recursive call to take all possible directions
                            find_new_paths(sub_path, other_face, faces, all_paths, path_function)

                        elif path_function(next_edge, other_face, current_path):
                            new_found_path = shift(current_path)
                            inverted_path = invert(current_path)
                            unique = lambda path: is_unique(path, all_paths)
                            if unique(new_found_path) and unique(inverted_path):
                                all_paths.append(new_found_path)
                    face_without_start.append(start)
        faces_without_current_face.append(face)


def remove_duplicates(faces, all_paths):
    '''
    :param faces:
    :type faces:
    :param all_paths:
    :type all_paths:
    :returns:

    '''
    paths = list(all_paths)
    for path in paths:
        for f in faces:
            counter = 0
            for e in f:
                if e in path:
                    counter += 1
            if counter == 3:
                all_paths.remove(path)
    return all_paths


def find_all_paths(faces):
    '''
    :param faces:
    :type faces:
    :returns:

    '''

    all_paths = []
    forward = lambda path, face, path_function: find_new_paths(path, face, faces, all_paths, path_function)
    for face in faces:
        for edge in face:
            forward([edge], face, path_finished_single)
            #forward([edge], face, pathFinishedDouble)
    all_paths = remove_duplicates(faces, all_paths)
    return all_paths

## Now that we have the paths, they need to be
## re- indexed so that matrices can be built from them.

def build_matrices(edge_paths, all_paths):
    '''
    :param edge_paths: list of face boundary orientations
    :type edge_paths: list
    :param all_paths: list
    :type all_paths: list

    Take the paths in the skeleton of the complement of the transverse curve
    And create matrices.
    '''
    #print 'edgePaths:',edgePaths
    #print 'AllPaths:',AllPaths[0].loops
    master_list = []

    # Allow paths to be referenced by face
    #print edgePaths

    for itr in range(len(edge_paths)):
        edge_paths[itr] = dict(edge_paths[itr])
    #print 'all_paths:',all_paths
    #print 'edge_paths:',edge_paths

    ordered_paths , mapped_paths = [],[]

    # Rescale path 0-len(path) for matrix
    for path in all_paths:
        ordered_path = list(np.sort(path))
        ordered_paths.append(ordered_path)
        mapped_paths.append(dict(zip(ordered_path,range(len(path)))))
    #print mapped_paths,'\n'
    #Create Matrices using details from edge paths.
    for Path, mapped_path in zip(all_paths, mapped_paths):
        #Value Matrix
        path_size = len(Path)
        shape = (path_size, 4)
        last = Path[len(Path)-1]
        M = np.zeros(shape)
        M2 = np.zeros(shape)
        M[:, 0] = np.array([path_size-1]+range(path_size-1))
        M[:, 2] = np.array(range(1,path_size)+[0])
        past_edges = dict()
        future_edges = dict()
        old_vertex = last

        itr = 1
        for vertex in Path:
            flag = False
            for path in edge_paths:
                keys = set(path.keys())
                if vertex in keys and old_vertex in keys:
                    past_edges[vertex] = path[vertex]
                    flag = True
                if vertex in keys and Path[itr % path_size] in keys:
                    if flag:
                        future_edges[vertex] = (path[vertex] + 2) % 4
                    else:
                        future_edges[vertex] = path[vertex]
                flag = False
            old_vertex = vertex
            itr += 1
        old_vertex = last

        itr = 1
        for vertex in Path:
            curr_vertex = vertex
            next_vertex = Path[itr % path_size]
            M[mapped_path[vertex], past_edges[vertex]] = mapped_path[old_vertex]
            M[mapped_path[vertex], future_edges[vertex]] = mapped_path[next_vertex]
            old_vertex = curr_vertex
            itr += 1

        # Sign matrix: Stand-alone function
        M = fix_matrix_signs(np.array([M, M2], dtype=int))
        master_list.append(M)
    # End while

    return master_list



def face_parse(alpha_edges):
    '''
    :param alpha_edges: set of faces with alpha edges.
    :typealpha_edgess:
    :returns: (Bridges, Isalnds, lengthCheck) Bridges : 4-sided regions; Islands : n > 4 - gons; lengthCheck : the number of alpha edges included.

    Separate set of all faces into bridges (4-sided regions) and
    islands (higher-sided regions) for distance calculator.

    '''
    bridges, islands = [], []
    length_check = []
    for pair in alpha_edges:
        if pair[0] == 4:
            bridges.append(pair)
        else:
            islands.append(pair)
        length_check.extend(list(pair[1]))
    return bridges, islands, len(length_check)

######					For Distance Extension 					#######


def connected(P1, P2):
    '''
    :param P1:
    :type P1:
    :param P2:
    :type P2:
    :returns:

    '''

    S1 = set(P1)
    S2 = set(P2)
    if S1.isdisjoint(S2) or (S1.issubset(S2) and S1.issuperset(S2)):
        return 0
    else:
        return 1


def share_edge(path1, path2):
    '''
    :param path1:
    :type path1:
    :param path2:
    :type path2:
    :returns:

    '''
    if not set(path1).isdisjoint(path2):
        return 0, -1
    else:
        intersection_set = set(path1) & set(path2)
        numshared = len(intersection_set)
        if numshared != 1:
            return 0, -1 #Then they share too much!
        else:
            shared_item = intersection_set.pop()
    return 1, path2.index(shared_item)


def find_combined_paths(all_paths, M_library):
    '''
    :param all_paths:
    :typeall_pathss:
    :param M_library:
    :typeM_libraryy:
    :returns:
    '''
    list_of_connected = []
    path_library = dict(zip(range(len(all_paths)), all_paths))
    index1 = 0
    for path1 in all_paths:
        index2 = 0
        for path2 in all_paths:
            if share_edge(path1, path2)[0]:
                list_of_connected.append((M_library[index1],
                            M_library[index2]))
            index2 += 1
        index1 += 1

    return list_of_connected


def faces_share_two_edges(faces):
    '''
    :param faces:
    :type faces:
    :returns:

    '''

    distance_three_flag = 0

    for face in faces:
        faces_without_face = list(faces)
        faces_without_face.remove(face)
        for other in faces_without_face:
            if len(set(face) & set(other)) >1: distance_three_flag = 1

    return distance_three_flag



def edges(M):
    '''
    :param M: the matrix representing a pair of curves
    :type M: numpy.array(dtype=float64) of shape (2,n, 4)
    :returns: (allFaces, edges) allFaces: tuple of faces including size and set of alpha-edges which bound them; edges: same as allFaces, exceincluding orientation of boundary edges.
    '''
    #print 'called edges'
    # The em list is needed to hold the tuples of (faceLength, faceEdges)
    all_faces = []
    # INV: Number of faces found.
    faces = 0

    num_rows, num_cols = M.shape[1:3] #num_rows, num_cols

    old_vertices = [] ##list of previous paths
    bigonFlag = 0
    # Bigons are unwanted structures.
    Paths = []  # alpha - edge paths.
    facesTemp = dict()
    for i,j in product(range(num_rows),range(num_cols)):
        #Set of edges associated with face
        tr = 1
        face = set()
        if faces == num_rows:
            break  # upper bound on possible no. faces
        # Start position in matrix: returning to this means a face has been
        # enclosed
        io =i
        jo=j

        found = 0 #exit condition
        # Number of edges for face. Keeps track of vector solution
        edges=0
        pathTemp = [];

        # Begin traversal
        while not found:

            gval = int(M[0,i,j]);
            #current value at index gives next vertex/row
            #value check
            arr1 = M[0,gval,:] == i%num_rows
            arr2 = M[1,gval,:] != M[1,i,j] #sign check
            i_next = gval
            alpha = (M[0,i,0]+1)%num_rows
            i= i_next
            new = np.where(arr1 & arr2)[0]
            #new = np.intersect1d(arr1[0],arr2[0],assume_unique=True)
            #ENS: val and sign correct
            ind = np.where(new%2==j%2) #ENS: beta->beta, alpha->alpha
            j_next = (int(new[ind])+1)%num_cols #Always move clockwise
            j_old = j
            j = j_next
            if (i,j) in old_vertices:
                break
            old_vertices.append((i,j))
            edges += 1

            alpha_new =( M[0,i,0]+1) % num_rows;
            shift = (alpha_new - alpha)

            if (shift==1 and j%2 == 1) or (shift == 1-num_rows and alpha_new==0 and j%2==1):
                face.add(alpha_new)
                pathTemp.append((alpha_new,(j)%num_cols))
            elif (shift==-1 and j%2==1 ) or (shift == num_rows-1 and alpha ==0  and j%2==1):
                face.add(alpha)
                pathTemp.append((alpha,(j)%num_cols))

            if (i,j)==(io,jo):
                facesTemp[edges] = facesTemp.get(edges,0) +1
                if edges==2:
                    bigonFlag = 1
                if not bigonFlag:
                    Paths.append(pathTemp)
                    faces += 1
                    found = 1
                    all_faces.append((edges,face))
    #from sys import stderr
    #stderr.write('all_faces '+str(all_faces)+'\n')
    #stderr.write('Paths '+str(Paths))
    return all_faces, Paths

def boundary_count(M):
    '''
    :param M: the matrix representing a pair of curves
    :type M: numpy.array(dtype=float64) of shape (2,n, 4)
    :returns: (faces, bigon) faces: number of faces; bigon: 1 iff a bigon is found

    'Simply' count the number of faces bounded by two filling curves on a surface.
    Curves must be encoded as matrix according to vertices of intersection and
    associated orientation.


    '''

    faces = 0
    numrows, numcols = M.shape[1:3] #num_rows, num_cols
    oldEdges = [ ] ##list of previous edge paths
    bigonFlag =0

    for i,j in product(range(numrows),range(numcols)):
        # upper bound on possible no. faces is number of vertices
        if faces==numrows: break

        io =i
        jo=j
        #First matrix element; will go to vertex M[0,i,j]

        found = 0 #Exit condition
        pathLength=0 # Keep track of path length

        while not found:
            gval = int(M[0,i,j]);
            #current value at index gives next vertex/row

            arr1 = np.where(M[0,gval,:] == i%numrows) #value check
            arr2 = np.where(M[1,gval,:] == -M[1,i,j]); #sign should flip +/-
            i = gval # Go to next vertex/row
            new = np.intersect1d(arr1[0],arr2[0],assume_unique=True)
            #ENS: val and sign correct

            ind = np.where(new%2==j%2)
            #ENS: beta->beta, alpha->alpha

            j = (int(new[ind])+1)%numcols
            #Always move clockwise - to next edge

            if (i,j) in oldEdges: break
            oldEdges.append((i,j))
            # To save work and not go on old paths.
            # Also so we don't count faces twice...
            pathLength += 1
            # The path length is the number of edges traversed in the current face.

            if (i,j)==(io,jo):
                if pathLength==2: bigonFlag = 1;
                # Two edges to a face --> bigon
                faces += 1
                found = 1 #INV: found = 1 -> has found a bdy curve
    return faces, bigonFlag

def vector_solution(edges):

    solution = dict()

    for face in edges:
        if face[0] not in solution.keys():
            solution[face[0]] = 1
        else:
            solution[face[0]] += 1

    return solution

def genus(M, euler=0, boundaries = 0):
    '''
    :param M: the matrix representing a pair of curves
    :type M: numpy.array(dtype=float64) of shape (2,n, 4)
    :param euler: 0 if euler characteristic not needed, else 1
    :type euler: int

    :returns: (g,X): g: genus; X : euler characteristic

    Compute the genus of the span of a curve pair, i.e. the minimal genus surface
    on which they fill.

    '''
    V = M.shape[1] # vertices
    P, bigon = boundary_count(M)  # Polygons in the complement of curve pair
    #if bigon is 1: P -= 1 # pull away one bigon
    #Euler characteristic (since edges = 2*vertices) is P - V
    # originally X = V-E+P
    X = P-V+boundaries

    # genus = 1 - 0.5*euler_characteristic
    Genus =  (2-X)/2
    returnVal = dict([(0,Genus),(1,(Genus,X))])
    # For return purposes only.

    if bigon: Genus -= 1 # Bigons steal genus; this gives it back.

    return returnVal[euler]

def test_collection(matrix_list, original_genus):
    '''
    :param matrix_list:
    :typematrix_listt: list of numpy.array(dtype=float64)
    :param original_genus:
    :typeoriginal_genuss:
    :returns: list of booleans and a dictionary containing the matrices without bigons.

    Test a collection of matrices to see if they fill a given genus.
    Note: Matrices with bigons are still counted in the calculation, but are not returned.
    '''
    genusCollection = []
    index = 0
    matrixLibrary = dict()
    for M in matrix_list:

        bigon = boundary_count(M)[1]
        Genus = genus(M)
        matrixLibrary[index] = M
        index += 1
        if bigon:
            Genus  = 0 # If it has a bigon, it should automatically fail

        genusCollection.append(Genus)
    # Test if all fill and thus distance g.t. 3
    test = [Genus == original_genus for Genus in genusCollection]
    return test, matrixLibrary

def fourgonTest(F4, Fn):
    '''
    :param F4:
    :type F4:
    :param Fn:
    :type Fn:
    :returns: boolean: True if Fn is a 4-gon, False otherwise

    '''

    for islandFace in Fn:
        for bridgeFace in F4:
            if len(islandFace[1] & bridgeFace[1]) >= 2: return True
    return False

def Three(M, allPaths, ext=0, originalGenus = False, boundaries = False, edges = False):
    '''
    :param M:
    :type M:
    :param allPaths:
    :type allPaths:
    :param ext:
    :type exr:
    :returns: 1 if the curve pair is distance three and 0 otherwise.

    '''

    three, matrixLibrary = 0, dict()

    if not originalGenus:
        originalGenus = genus(M)
    if not boundaries:
        F0, bigon = boundary_count(M)
    else:
        F0, bigon = boundaries
    if not edges:
        #Calculate face alpha edges and alpha edge paths
        Faces, edgePaths = edges(M)
    else:
        Faces, edgePaths = edges

    Bridges, Islands, lengthCheck = face_parse(Faces)

    # Bridges are faces bounded by four edges.
    # Islands are bounded bt more than four, say six or eight...
    bridgeFaces = [list(face[1]) for face in Bridges]
    islandFaces = [list(face[1]) for face in Islands]

    ############# Quick and dirty checks for distance three #############
    if F0 == 1 :
        three = 1
    #	print '''The complement contains a polygon which shares an edge with
    #			itself, and so is distance 3. '''


    # If any face is larger than the number of vertices, it will definitely
    # Share an edge with itself. Therefore distance three.
    faceSizeTest = [face[0]  > M.shape[1] for face in Faces]
    if True in faceSizeTest:
        three = 1
    #	print '''The complement contains a polygon which shares an edge with
    #			itself, and so is distance 3. '''


    if  fourgonTest(Bridges, Islands):
        three = 1


    if lengthCheck != 2*M.shape[1]:
        three = 1
    #	print '''The complement contains a polygon which shares an edge with
    #			itself, and so is distance 3. '''
    # See function faceParse
    # If this is true, then some face is sharing an edge with itself.
    # It can't be a four-gon, since that would mean we have a multi-curve.
    # So it must be an island, which means that there is a curve
    # in the complement which is distance two. Thus the original pair is
    # Distance three.

    if faces_share_two_edges(islandFaces):
        three = 1
    #	print ''' Found two faces that share multiple edges: A curve that
    #			  intersects the non-reference curve only two times has been
    #			  found. This curve cannot fill and so the pair is distance 3.
    #		  '''

    # Means there is a path of length two.
    # Since a curve pair with two intersections cannot fill on any genus >1 ,
    # The two curves will be distance three.

    #Find linking edge paths in "mesh"
    faces  = list(bridgeFaces)
    faces += islandFaces

    #Build paths into curves and intersect with alpha
    matrixList = build_matrices(edgePaths, allPaths)

    genusTest, matrixLibrary = test_collection(matrixList, originalGenus)
    if three != 1:
        three = 1 if False in genusTest else 0

    # Output
    returnVals = [three, matrixLibrary]
    return returnVals

def ladder_convert(ladder_top, ladder_bottom):
    #print ladderTop, ladderBottom
    n = len(ladder_top)
    newTop = [' ']*n
    newBottom = list(newTop)

    for j in range(1, n+1):

        if j in ladder_top and j in ladder_bottom:
            newTop[ladder_top.index(j)]  = ladder_bottom.index(j)
            newBottom[ladder_bottom.index(j)] = ladder_top.index(j)

        elif j in ladder_top:
            ladderTopTemp = list(ladder_top)
            indices = [ladder_top.index(j)]; ladderTopTemp[indices[0]] = None#ladderTopTemp.remove(j)
            indices.append(ladderTopTemp.index(j))
            newTop[indices[0]] = indices[1]
            newTop[indices[1]] = indices[0]

        elif j in ladder_bottom:
            ladderBottomTemp = list(ladder_bottom)
            indices = [ladder_bottom.index(j)]; ladderBottomTemp[indices[0]] = None #ladderBottomTemp.remove(j)
            indices.append(ladderBottomTemp.index(j))
            newBottom[indices[0]] = indices[1]
            newBottom[indices[1]] = indices[0]

    return newTop, newBottom


def ladder_is_multicurve(top, bottom):

    n = len(top)

    j0 = top[0]
    counter = 1

    j = bottom[0]
    bottom[0] = None
    oldIndex = 0
    while j != j0:

        old_j = j
        if j in top:
            nextIndex = top.index(j)
            j = bottom[nextIndex]

            if None in top:
                top[top.index(None)] =  old_j
            elif None in bottom:
                bottom[bottom.index(None)] =  old_j

            bottom[nextIndex] = None

        elif j in bottom:
            nextIndex = bottom.index(j)
            j = top[nextIndex]

            if None in top:
                top[top.index(None)] =  old_j
            elif None in bottom:
                bottom[bottom.index(None)] =  old_j

            top[nextIndex] = None

        counter += 1

    if None in top:
        top[top.index(None)] =  j
    elif None in bottom:
        bottom[bottom.index(None)] =  j


    return 1 if counter != n else 0

def matrix_is_multicurve(beta):

    top = beta[0]
    bottom = beta[1]
    index = 0
    j = top[index]
    start = 100
    counter = 0

    while j != start:
        counter += 1
        start = top[0]

        if top[j] == index :
            next_index = j
            j = bottom[j]
            index = next_index

        elif bottom[j] == index:
            next_index = j
            j = top[j]
            index = next_index

        if top[j] == bottom[j]: return True
        #print 'j', j,'next_index: ',  index

    return False if counter == len(top) else True


def test_permutations(original_ladder):
    distance4 = []
    distance3 = []
    #from curvepair import CurvePair
    ladder = deepcopy(original_ladder)
    if not original_ladder is None:
        for i in range(len(ladder[0])):
            if not ladder_is_multicurve(*ladder):
                perm = CurvePair(*ladder)
            else:
                perm = None
            if not perm is None :
                if perm.distance is 4:
                    distance4.append(deepcopy(perm))
                else:
                    distance3.append(deepcopy(perm))
            else: pass
            first_vertex = ladder[0].pop(0)
            ladder[0].append(first_vertex)

        if len(distance4) == 0:
            print ' Found no distance four permutations of the ladder. '

        if distance3:
            print 'Distance 3 single curves: '
            for curve in distance3:
                print 'top   : ', curve.ladder[0]
                print 'bottom: ', curve.ladder[1]
        if distance4:
            print 'Distance 4+ single curves: '
            for curve in distance4:
                print 'top   : ', curve.ladder[0]
                print 'bottom: ', curve.ladder[1]

        return distance4
    else:
        print "You didn't give me a ladder! "
        return []


def test_perms(original_ladder):
    distance4 = []
    distance3 = []
    ladder_to_perm = deepcopy(original_ladder)
    #from curvepair import CurvePair
    if not original_ladder is None:
        for i in range(len(ladder_to_perm[0])):
            if not ladder_is_multicurve(*ladder_to_perm):
                perm = CurvePair(*deepcopy(ladder_to_perm))
            else:
                perm = None
            if not perm is None :
                if not perm.distance is 3:
                    distance4.append(perm)
                else:
                    distance3.append(perm)
            else: pass

            first_vertex = ladder_to_perm[0].pop(0)
            ladder_to_perm[0].append(first_vertex)
    return distance3, distance4

#4+2-3-5-1+

#[1,2,3,2,4]
#[5,3,4,1,5]

#1-6+4-2+5-7+3+

#[7,4,7,2,4,2,6]
#[1,3,6,3,5,1,5]

#2+5-7+3+1-6+4-

#[4,1,4,6,1,6,3]
#[5,7,3,7,2,5,2]
def ladder_to_cycle(ladder_top, ladder_bottom):
    locations = {arc: {'top': [], 'bottom': []} for arc in set(ladder_top + ladder_bottom)}
    for loc, varc in enumerate(ladder_top):
        locations[varc]['top'].append(loc)

    for loc, varc in enumerate(ladder_bottom):
        locations[varc]['bottom'].append(loc)

    n = len(ladder_top)
    cycle = ''
    #arbitrarily orient them positively
    orientation = '+'
    current = 'top'
    start = 1
    prev_loc = (ladder_top+ladder_bottom).index(start) % n
    for i in range(n):
        #get the top and bottom of the current vertex
        top = locations[start]['top']
        bottom = locations[start]['bottom']

        # if there is an endpoint on both the top and bottom,
        # then orientation is preserved and the location is simply
        # whatever hasn't already been used. We switch side of the
        # ladder accordingly
        if top and bottom:
            # switch side of ladder
            current = 'top' if current == 'bottom' else 'bottom'
            # preserve orientation
            orientation = '-' if orientation == '-' else '+'
            # get the locations avaiable there
            locs = locations[start][current]
            cycle += str(locs[0]+1) + orientation
            if current == 'bottom':
                start = ladder_top[locs[0]]
            if current == 'top':
                start = ladder_bottom[locs[0]]
            prev_loc = locs[0]
            current = 'top' if current == 'bottom' else 'bottom'
        elif not top:
            current = 'bottom'
            orientation = '-' if orientation == '+' else '+'
            locs = locations[start][current]
            t = set(locs)
            t.remove(prev_loc)
            current_loc = t.pop()
            cycle += str(current_loc+1)+orientation
            start = ladder_top[current_loc]
            prev_loc = current_loc
            current = 'top'
        elif not bottom:
            current = 'top'
            orientation = '-' if orientation == '+' else '+'
            locs = locations[start][current]
            t = set(locs)
            t.remove(prev_loc)
            current_loc = t.pop()
            cycle += str(current_loc+1)+orientation
            start = ladder_bottom[current_loc]
            prev_loc = current_loc

            current = 'bottom'
    return cycle


def cycle_to_ladder(cycle_rep):
    arcs = [int(i) for i in re.split('[-+]', cycle_rep)[:-1]]
    n = len(arcs)
    signs = re.split('[0-9]+', cycle_rep)[1:]
    top = [0 for i in range(len(arcs))]
    bottom = [0 for i in range(len(arcs))]
    ladder = [top, bottom]
    
    ladder_index = 0
    for i in range(1, len(arcs)+1):
        current_sign = signs.pop(0)
        current_v = arcs.pop(0)
        if current_sign == '+':
            ladder[0][current_v-1] = i 
            if i == 1:
                ladder[1][current_v-1] = n#((i - 2) % n)
            else:
                ladder[1][current_v-1] = ((i - 1) % n)

        if current_sign == '-':
            ladder[1][current_v-1] = i 
            if i == 1:
                ladder[0][current_v-1] = n#((i - 2) % n)
            else:
                ladder[0][current_v-1] = ((i - 1) % n)


    return ladder


#import numpy as np
#from curves import fix_matrix_signs, boundary_count, genus, ladder_convert, vector_solution, edges, Three
from graph import Graph


class CurvePair:
    '''
    ladder = None
    beta = None
    top = []
    bottom = []
    n = 0
    matrix = None
    boundaries = None
    genus = None
    edges = None

    solution = None
    distance = 0
    loops = []
    '''
    def __init__(self, top_beta, bottom_beta, dist=1, conjectured_dist=3,recursive=False):

        is_ladder = lambda top, bottom: not (0 in top or 0 in bottom)

        if is_ladder(top_beta, bottom_beta):
            self.ladder = [top_beta, bottom_beta]
        else:
            self.ladder = None

        if is_ladder(top_beta, bottom_beta):
            self.beta = ladder_convert(top_beta, bottom_beta)
            self.top = self.beta[0]
            self.bottom = self.beta[1]
        else:
            self.top = top_beta
            self.bottom = bottom_beta
            self.beta = [self.top, self.bottom]

        self.n = len(self.top)

        self.matrix = np.zeros((2,self.n,4))
        self.matrix[0,:,0] = [self.n-1] + range(self.n-1)
        self.matrix[0,:,1] = self.top
        self.matrix[0,:,2] = range(1,self.n) +[0]
        self.matrix[0,:,3] = self.bottom

        self.matrix = fix_matrix_signs(self.matrix)

        self.boundaries = boundary_count(self.matrix)
        self.genus = genus(self.matrix)
        self.edges = edges(self.matrix)
        #self.arc_boundary = self.edges[1]
        self.solution = vector_solution(self.edges[0])

        self.loops = []
        self.dist = dist
        self.conjectured_dist = conjectured_dist
        self.computed_distance = False
        self.recursive = recursive
    
    @property
    def distance(self):
        if self.computed_distance == False:
            if self.dist is 1:
                graph = Graph(self.edges, rep_num=self.conjectured_dist-2)
                graph.compute_loops(self.n, self.genus)
                self.loops = graph.gammas
                #stderr.write(str(self.loops)+'\n')
                self.computed_distance, self.loop_matrices = self.compute_distance(self.matrix, self.loops, recursive=self.recursive)

            else:
                self.computed_distance = None
            return self.computed_distance
        else:
            return self.computed_distance


    def __repr__(self):
        return str(self.ladder[0])+'\n'+str(self.ladder[1])+'\n'

    def compute_distance(self, M, all_paths,recursive=True):
        '''
        :param M: the matrix
        :type M:
        :param all_paths:
        :type all_paths:
        :returns: dist: the distance if three/four, or 'Higher' if dist is > 4.

        Computes the distance between the two curves embedded in the matrix.
        If this distance is three, tries to use simple paths to extend the distance
        in a different direction. If this fails, simply returns three;
        else it prints a curve that is distance four from alpha.

        '''

        dist_is_three, lib = Three(M, all_paths, originalGenus=self.genus, boundaries=self.boundaries, edges=self.edges)
        dist = 3 if dist_is_three  else 'at least 4!'
        if dist == 3:
            return dist, lib
        else:
            if recursive:
                geodesic_distances = []
                for k, matrix in lib.iteritems():
                    #stderr.write(str(matrix))
                    if np.array_equal(matrix, self.matrix):
                        continue
                    elif self.solution == CurvePair(matrix[0, :, 1], matrix[0, :, 3],0).solution \
                            and len(self.matrix[0]) == len(matrix[0]):
                        continue
                    cc = CurvePair(matrix[0, :, 1], matrix[0, :, 3])
                    #stderr.write(str(k)+": "+str(cc.distance)+'\n')
                    geodesic_distances.append(cc.distance)
                    #print 'computed curve',k,'!'
                #print '\n'
                return min(set(geodesic_distances)) + 1, lib
            else:
                return dist, lib




from micc.curves import CurvePair, edges, build_matrices, fix_matrix_signs,Three
from micc.graph import Graph
from line_profiler import LineProfiler


def do_profile(follow=[]):
    def inner(func):
        def profiled_func(*args, **kwargs):
            try:
                profiler = LineProfiler()
                profiler.add_function(func)
                for f in follow:
                    profiler.add_function(f)
                profiler.enable_by_count()
                return func(*args, **kwargs)
            finally:
                profiler.print_stats()
        return profiled_func
    return inner


def get_number():
    for x in xrange(5000000):
        yield x

@do_profile(follow=[get_number])
def expensive_function():
    for x in get_number():
        i = x ^ x ^ x
    return 'some result!'

#result = expensive_function()


def get_dist(top,bot):
    return CurvePair(top,bot).distance

#@do_profile(follow=[get_dist,Graph.compute_loops, Graph.iter_loop_dfs,Graph.faces_share_edges,Graph.loop_dfs, edges, build_matrices,fix_matrix_signs])
@do_profile(follow=[get_dist,CurvePair.compute_distance, Three, Graph.compute_loops, Graph.iter_loop_dfs])
def profile_distance(top,bot):
    return get_dist(top,bot)


#print profile_distance([6, 7, 8, 9, 10, 11, 13, 14, 15, 16, 17, 1, 2, 3, 4, 17, 5],
#                [1, 2, 12, 13, 14, 15, 16, 5, 6, 7, 8, 9, 10, 11, 12, 3, 4])
#print profile_distance([23, 22, 1, 2, 3, 4, 5, 22, 23, 24, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 24],
#                       [1, 2, 3, 4, 19, 20, 21, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 21, 20, 19, 5])
print profile_distance([1,2,3,4,12,10,14,15,9,13,11,12,4,5,6,7,21,22,23,24,16,17,18,19],[24,1,2,3,13,9,15,16,8,14,10,11,5,6,7,8,20,21,22,23,17,18,19,20])
#print profile_distance([1,11,3,27,8,15,7,24,29,10,2,12,4,21,19,17,24,14,6,23,28,9,16,25,13,5,20,18,16],[29,10,2,26,7,14,6,23,28,9,1,11,3,22,20,18,25,13,5,22,27,8,15,26,12,4,21,19,17])
#print profile_distance([1,2,3,4,12,10,14,15,9,13,11,12,4,5,6,7,21,22,23,24,16,17,18,19],[24,1,2,3,13,9,15,16,8,14,10,11,5,6,7,8,20,21,22,23,17,18,19,20])



#!/usr/bin/env python
import curves as c
from curves import CurvePair, cycle_to_ladder, ladder_to_cycle
import readline

class CLI:

    perm_dict = {}

    def __init__(self):
        self.commands = {
                'genus':self.get_genus,
                'faces':self.get_faces,
                'perm':self.get_perms,
                'distance':self.get_distance,
                'curves':self.get_curves,
                'matrix':self.get_matrix,
                'help':self.get_help,
                'change':self.run,
                'exit':self.quit,
                'quit':self.quit,
                'bye':self.quit
                }

        self.perms = dict()
        self.ladder = []

    def is_multi(self,topRow,bottomRow):
        if 0 in topRow or 0 in bottomRow:
            multi = c.matrix_is_multicurve([topRow, bottomRow])
        else:
            multi  = c.ladder_is_multicurve(topRow, bottomRow)
        return multi

    def correct_input(self,ladder):
        while ' ' in ladder: ladder.remove(' ')
        while  '' in ladder: ladder.remove('')
        for string in ladder:
            string = list(string)
            while ' ' in string: string.remove(' ')
            string = str(string)

        return ladder

    def made_mistake(self,top, bottom):

        if len(top) == 0 or len(bottom) == 0: return True

        indices =  dict(zip(set(top+bottom), 2*len(top)*[0]))
        for val in top:
            indices[val] += 1
        for val in bottom:
            indices[val] += 1
        tooMany = [ x != 2 for x in indices.values() ]

        if not True in tooMany and len(top) != len(bottom):
            return True

        return True in tooMany

    def process_input(self):
        valid_input = False
        ret = False
        while not valid_input:
            input = raw_input("What would you like to calculate? ")
            if input in self.commands.keys():
                ret = self.commands[input]()
                valid_input = True
            else:
                raw_input("Sorry, I didn't quite catch that. Press enter and try again. ")
        return ret



    def quit(self):
        quit()


    def get_genus(self):
        print "Genus: ",self.curve.genus
        return False

    def get_faces(self):
        curve = self.curve
        print '%s faces  with %s bigons'%tuple(curve.boundaries)
        print 'Vector solution: ', self.curve.solution
        for face in curve.edges[0]:
            print tuple(face[1])
        return False

    def get_distance(self):
        print 'Distance: ', self.curve.distance
        return False

    def shear(self):
        self.perms = dict()
        for index, perm in enumerate(c.test_permutations(self.ladder),start=1):
            self.perms[index] = perm
            print '\n'
            print 'Curve', index, ' Distance: ', perm.distance
            print perm.ladder[0]
            print perm.ladder[1]
        if raw_input('would you like to keep going? ') == 'no': quit()
        return False


    def get_perms(self):
        self.perms = dict()

        for index, perm in enumerate(c.test_permutations(self.curve.ladder),start=1):
            self.perms[index] = perm
            print '\n'
            print 'Curve', index, ' Distance: ', perm.distance
            print perm.ladder[0]
            print perm.ladder[1]
        return False

    def get_curves(self):
        view = raw_input("Would you like to view the vertex paths,"+\
                                    " the corresponding matrices, or both? ")

        for itr in range(len(self.curve.loops)):

            loop = self.curve.loops[itr]
            matrix = self.curve.loop_matrices.values()[itr]
            Genus = c.genus(matrix)

            if view=='paths' or view=='both':
                print 'Path', loop

            if view=='matrices' or view== 'both':
                print 'Matrix: \n',matrix[0]

            print 'Curve genus: ', Genus,'\n'
        return False

    def get_matrix(self):
        print self.curve.matrix
        return False

    def clear(self):
        self.perms = dict()
        self.curve = None
        self.topRow = []
        self.bottomRow = []
        return False

    def get_help(self):
        print ''' Welcome to curvePair.
        \n With this program, by supplying a pair of curves you can determine:
        \n 1. The genus of the surface on which the curves fill (say 'genus')
        \n 2. The number of boundary components in the complement of the curve pair on such a surface , and the vector solution (say 'faces')
        \n 3. The distance between the two curves in the curve complex, up to distance four (say 'distance')
        \n 4. The curves in the complement of the non-reference curve that were used for calculating distance (say 'curves')
        \n 5. To work with a different curve pair, say 'change'
        \n 6. To see the matrix associated with the curve, say 'matrix'
        \n 7. If a ladder is entered, can see the different curves that result from permutations of the identifications in that ladder (say perm)
        \n
        \n ---------------begin curve entry help -------------
        \n Instructions for entering curves: Using an orientation and reference
        \ncurve of your choice, please number the intersections of the two curves
        \n(starting from 1). Then trace out the non-referential curve, identifying
        \nwhich of the intersections are connected. Keep track of whether each
        \nconnection is above or below the reference curve. The program will ask
        \nfor a comma-separated list of these connections, starting from the
        \nones which come from above and then the ones which come from below.
        \nFor example: if the first vertex (1) is connected to the fifth (5) from
        \nabove, then the first number you will enter is a 5, and so on.

        \nAlternatively, you can number the identifications(edges) from 1 to
        \n the number of edges and enter those that are on the top
        \n and those that are on the bottom.
        \n ------------------ end help ----------------------

        \n Note: say 'done' to exit
        '''

        return False


    def run(self):
        self.clear()
        self.get_help()
        cycle_or_ladder = raw_input('Would you like to describe curves as a ladder or a cycle?\n (Type \'cycle\' or \'ladder\'): ')

        valid = False
        valid_exit_status = None
        while not valid:
            if cycle_or_ladder == 'ladder':
                self.topRow = raw_input('Input top indentifications: ')
                if self.topRow == 'exit' or self.topRow == 'quit': quit()
                self.topRow = self.topRow.split(',')
                self.topRow = self.correct_input(self.topRow)
                self.topRow = [int(num) for num in self.topRow]

                self.bottomRow = raw_input('Input bottom indentifications: ')
                self.bottomRow = self.bottomRow.split(',')
                self.bottomRow = self.correct_input(self.bottomRow)
                self.bottomRow = [int(num) for num in self.bottomRow]

                print 'Input (as a cycle):'
                print ladder_to_cycle(self.topRow, self.bottomRow)
                print 'Input (as a ladder): '
                print self.topRow
                print self.bottomRow
                valid = not self.made_mistake(self.topRow,self.bottomRow)

                if not valid:
                    print "There seems to be an error in your entry. Please try again. "
            if cycle_or_ladder == 'cycle':
                cycle = raw_input('Input cycle: ')
                print 'Input (as a cycle):'
                print cycle
                ladder = cycle_to_ladder(cycle)
                self.topRow = ladder[0]
                self.bottomRow = ladder[1]
                print 'Input (as a ladder):'
                print self.topRow
                print self.bottomRow
                valid = not self.made_mistake(self.topRow,self.bottomRow)
                if not valid:
                    print "There seems to be an error in your entry. Please try again. "

        if not self.is_multi(self.topRow, self.bottomRow):
                self.curve = CurvePair(self.topRow, self.bottomRow)
                print 'Note: if you permute a curve with \'perm\', the sheared curves will be lost.'

        else:
            self.ladder = [self.topRow, self.bottomRow]
            ans = raw_input('Would you like to shear this multicurve? ')
            while ans != 'yes' and ans != 'no':
                ans = raw_input('I didn\'t understand. Would you like to shear this multicurve? ')
            if ans == 'yes':
                self.shear()
                self.run()
            elif ans == 'no':
                print 'Better luck next time. '
                quit()

        while not valid_exit_status:
             valid_exit_status = self.process_input()

        return True

CLI().run()



import multiprocessing as mp
from graph import shift
from time import sleep

def count(adj_node, adj_list):
    return adj_list.count(adj_node)

def loop_dfs( current_node,  start_node,  graph,  current_path,  nodes_to_faces):
    if len(current_path) >= 3:
        path_head_3 = current_path[-3:]
        previous_three_faces = [set(nodes_to_faces[edge]) for edge in path_head_3]
        intersection_all = set.intersection(*previous_three_faces)
        if len(intersection_all) == 2:
            return []

    if current_node == start_node:
        #stderr.write("Found one! \n")
        #all_loops.append(shift(list(current_path)))
        return [shift(list(current_path))]

    else:
        loops = []
        for adjacent_node in set(graph[current_node]):
            if count(adjacent_node, current_path) < 1:
                current_path.append(adjacent_node)
                graph[current_node].remove(adjacent_node)
                graph[adjacent_node].remove(current_node)
                loops += list(loop_dfs(adjacent_node, start_node, graph, current_path, nodes_to_faces))
                graph[current_node].append(adjacent_node)
                graph[adjacent_node].append(current_node)
                current_path.pop()
        return loops


def dfs_partial( current_node,  start_node,  graph,  current_path,  nodes_to_faces):

    if len(current_path) >= 3:
        path_head_3 = current_path[-3:]
        previous_three_faces = [set(nodes_to_faces[edge]) for edge in path_head_3]
        intersection_all = set.intersection(*previous_three_faces)
        if len(intersection_all) == 2:
            return []

    if current_node == start_node:
        return [shift(list(current_path))]

    if len(current_path) >= graph.keys()/4:
        return  [('work', current_node,  start_node,  graph,  current_path,  nodes_to_faces)]

    else:
        loops = []
        for adjacent_node in set(graph[current_node]):
            if count(adjacent_node, current_path) < 1:
                current_path.append(adjacent_node)
                graph[current_node].remove(adjacent_node)
                graph[adjacent_node].remove(current_node)
                loops += list(loop_dfs(adjacent_node, start_node, graph, current_path, nodes_to_faces))
                graph[current_node].append(adjacent_node)
                graph[adjacent_node].append(current_node)
                current_path.pop()
        return loops


class PGraph:

    class Master:
        num_threads = 0
        workers = []

        def __init__(self):
            self.num_threads = mp.cpu_count()
            self.work_queue = mp.Queue()
            self.results_queue = mp.Queue()
            self.workers = [PGraph.Worker(self.work_queue, self.results_queue) for i in range(self.num_threads+2)]

        def execute(self):
            #starts all workers, then begins searching for work for them to complete
            p = mp.Process(target=self.find_work())
            p.start()
            for worker in self.workers:
                worker.run()
            p.join()

        def find_work(self):
            pass

    class Worker:
        threads = []

        def __init__(self, work_queue, results_queue):
            self.results_queue = results_queue
            self.work_queue = work_queue

        def run(self):
            while not self.work_queue.empty():
                proc = mp.Process(target=loop_dfs, args=self.work_queue.get(0))
                proc.start()
                self.threads.append(proc)
            for thread in self.threads:
                thread.join()

    def __init__(self):
        pass



"""
This is a setup.py script generated by py2applet

Usage:
    python setup.py py2app
"""

from setuptools import setup
'''
from sys import setrecursionlimit
setrecursionlimit(10000)
APP = ['micc/cli.py']
DATA_FILES = []
OPTIONS = {'argv_emulation': True}

setup(
    app=APP,
    data_files=DATA_FILES,
    options={'py2app': OPTIONS},
    setup_requires=['py2app'],
)
'''

setup(name='MICC',
      version='0.1.5',
      author='Matt Morse, Paul Glenn',
      author_email='mjmorse@buffalo.edu, paulglen@berkeley.edu',
      packages=['micc', 'micc.tests'],
      url='http://micc.github.io',
      download_url='https://github.com/micc/micc',
      license='LICENSE',
      description='An implementation of the Birman-Margalit-Menasco Theorem, to be used in a experimental, exploratory manner.',
      long_description=open('README.rst').read(),
      install_requires=[
          "numpy== 1.9.1",
          "nose >= 1.3.1",
          "rednose"
      ],
      test_suite="nose.collector")
